package com.sun.org.apache.xml.internal.security.utils;

import com.sun.org.slf4j.internal.LoggerFactory;
import java.lang.reflect.Method;
import com.sun.org.apache.xml.internal.security.transforms.implementations.FuncHere;
import com.sun.org.apache.xpath.internal.Expression;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.SourceLocator;
import com.sun.org.apache.xml.internal.utils.PrefixResolver;
import com.sun.org.apache.xml.internal.utils.PrefixResolverDefault;
import org.w3c.dom.Document;
import com.sun.org.apache.xpath.internal.objects.XObject;
import javax.xml.transform.TransformerException;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import com.sun.org.apache.xpath.internal.XPathContext;
import com.sun.org.apache.xpath.internal.compiler.FunctionTable;
import com.sun.org.apache.xpath.internal.XPath;
import com.sun.org.slf4j.internal.Logger;

public class XalanXPathAPI implements XPathAPI
{
    private static final Logger LOG;
    private String xpathStr;
    private XPath xpath;
    private static FunctionTable funcTable;
    private static boolean installed;
    private XPathContext context;
    
    public XalanXPathAPI() {
    }
    
    @Override
    public NodeList selectNodeList(Node node, Node node2, String s, Node node3) throws TransformerException {
        return this.eval(node, node2, s, node3).nodelist();
    }
    
    @Override
    public boolean evaluate(Node node, Node node2, String s, Node node3) throws TransformerException {
        return this.eval(node, node2, s, node3).bool();
    }
    
    @Override
    public void clear() {
        this.xpathStr = null;
        this.xpath = null;
        this.context = null;
    }
    
    public static synchronized boolean isInstalled() {
        return XalanXPathAPI.installed;
    }
    
    private XObject eval(Node node, Node owner, String xpathStr, Node xpathExpressionContext) throws TransformerException {
        if (this.context == null) {
            (this.context = new XPathContext(owner)).setSecureProcessing(true);
        }
        xpathExpressionContext = ((xpathExpressionContext.getNodeType() == 9) ? ((Document)xpathExpressionContext).getDocumentElement() : xpathExpressionContext);
        PrefixResolverDefault namespaceContext = new PrefixResolverDefault(xpathExpressionContext);
        if (!xpathStr.equals(this.xpathStr)) {
            if (xpathStr.indexOf("here()") > 0) {
                this.context.reset();
            }
            this.xpath = this.createXPath(xpathStr, namespaceContext);
            this.xpathStr = xpathStr;
        }
        return this.xpath.execute(this.context, this.context.getDTMHandleFromNode(node), namespaceContext);
    }
    
    private XPath createXPath(String exprString, PrefixResolver prefixResolver) throws TransformerException {
        Object o = null;
        Class[] parameterTypes = { String.class, SourceLocator.class, PrefixResolver.class, Integer.TYPE, ErrorListener.class, FunctionTable.class };
        Object[] initargs = { exprString, null, prefixResolver, Integer.valueOf(0), null, XalanXPathAPI.funcTable };
        try {
            o = (Object)XPath.class.getConstructor(parameterTypes).newInstance(initargs);
        }
        catch (Exception e) {
            XalanXPathAPI.LOG.debug(e.getMessage(), e);
        }
        if (o == null) {
            o = new XPath(exprString, null, prefixResolver, 0, null);
            return o;
        }
        return o;
    }
    
    private static synchronized void fixupFunctionTable() {
        XalanXPathAPI.installed = false;
        if (new FunctionTable().functionAvailable("here")) {
            XalanXPathAPI.LOG.debug("Here function already registered");
            XalanXPathAPI.installed = true;
            return;
        }
        XalanXPathAPI.LOG.debug("Registering Here function");
        try {
            Method method = FunctionTable.class.getMethod("installFunction", new Class[] { String.class, Expression.class });
            if ((method.getModifiers() & 0x8) != 0x0) {
                method.invoke(null, new Object[] { "here", new FuncHere() });
                XalanXPathAPI.installed = true;
            }
        }
        catch (Exception e) {
            XalanXPathAPI.LOG.debug("Error installing function using the static installFunction method", e);
        }
        if (!XalanXPathAPI.installed) {
            try {
                XalanXPathAPI.funcTable = new FunctionTable();
                FunctionTable.class.getMethod("installFunction", new Class[] { String.class, Class.class }).invoke(XalanXPathAPI.funcTable, new Object[] { "here", FuncHere.class });
                XalanXPathAPI.installed = true;
            }
            catch (Exception e2) {
                XalanXPathAPI.LOG.debug("Error installing function using the static installFunction method", e2);
            }
        }
        if (XalanXPathAPI.installed) {
            XalanXPathAPI.LOG.debug("Registered class {} for XPath function 'here()' function in internal table", new Object[] { FuncHere.class.getName() });
        }
        else {
            XalanXPathAPI.LOG.debug("Unable to register class {} for XPath function 'here()' function in internal table", new Object[] { FuncHere.class.getName() });
        }
        return;
    }
    
    static {
        LOG = LoggerFactory.getLogger(XalanXPathAPI.class);
        fixupFunctionTable();
    }
}
